#ifndef FEBMAP_H
#define FEBMAP_H

#include <vector>
#include "mixturedensmap.h"

class FEBMAP : public MixtureDensityMAP {
public:
    int M;
    int* branchOrders;
    double* lambda;
    double* z;
    double* alpha;
    double* P;


    FEBMAP(std::vector<int> brOrders, int typeNum=1);
    ~FEBMAP ();

    void setInitialGuess (InitialGuess initial, Trace* trace, double trMean);

    void printSummary (std::ostream& os) const;
    void printAsMAP (std::ostream& os, const std::string& format) const;
    void printAsPH (std::ostream& os, const std::string& format) const;

    std::vector<double> getBranchMoments (int order) const;
    const double*       getBranchProbs () const { return alpha; }
    const double*       getBranchTransitionMatrix () const { return P; }
    int                 getNumOfBranches () const { return M; }
    const int*          getErlangOrders () const { return branchOrders; }
    const int           getNumOfErlangs () const { return M; }

};

#endif // FEBMAP_H
