#ifndef HERMAP_H
#define HERMAP_H

#include "mixturedensmap.h"
#include <vector>
#include <iostream>
#include <cmath>

// This class represents an MMAP having an ER-CHMM structure
class HErMAP : public MixtureDensityMAP {
public:
    int M;
    int* branchOrders;
    double* lambda;
    double* alpha;
    double* P;

    HErMAP (std::vector<int> brOrders, int typeNum=1);
    ~HErMAP ();

    void setInitialGuess (InitialGuess initial, Trace* trace, double trMean);

    inline double   getBranchDensity (double x, int branch) const {
                        double erlFact = lambda[branch];
                        for (int n=1; n<branchOrders[branch]; n++)
                            erlFact *= lambda[branch] * x / n;
                        return exp (-lambda[branch] * x) * erlFact;
                    }

    void printSummary (std::ostream& os) const;
    void printAsMAP (std::ostream& os, const std::string& format) const;
    void printAsPH (std::ostream& os, const std::string& format) const;

    std::vector<double> getBranchMoments (int order) const;
    const double*       getBranchProbs () const { return alpha; }
    const double*       getBranchTransitionMatrix () const { return P; }
    int                 getNumOfBranches () const { return M; }
    const int*          getErlangOrders () const { return branchOrders; }
    const int           getNumOfErlangs () const { return M; }
};

#endif // HERMAP_H
