#ifndef MIXTUREDENSMAP_H
#define MIXTUREDENSMAP_H

#include <vector>
#include <string>
#include <iostream>
#include "statprovider.h"

enum InitialGuess { IGLinear, IGExponential, IGKMeans, IGRandom, IGCombined };

class Trace;
class MixtureDensityMAP : public StatProvider {

protected:

    void printVector (std::ostream& os, std::vector<double> vec, std::string format) const;
    void printMatrix (std::ostream& os, std::vector<std::vector<double> > mat, std::string format) const;

public:
    int C;
    double logli;
    int iter;
    std::string source;

    MixtureDensityMAP(int typeNum=1);
    virtual ~MixtureDensityMAP() {}

    virtual void setInitialGuess (InitialGuess initial, Trace* trace, double trMean) = 0;

    void setSource (const std::string& src) { source = src; }

    std::vector<double>                 getMoments (int upToOrder) const;
    std::vector<double>                 getBackwardMoments (int upToOrder, int k) const;
    std::vector<double>                 getForwardMoments (int upToOrder, int k) const;
    std::vector<std::vector<double> >   getJointMoments (int upToOrder) const;
    std::vector<std::vector<double> >   getJointMomentsOfType (int upToOrder, int p) const;
    std::vector<double>                 getAcf (int maxLag) const;
    std::vector<double>                 getCrossAcf (int maxLag, int k, int p) const;

    virtual void printSummary (std::ostream& os) const = 0;
    virtual void printAsMAP (std::ostream& os, const std::string& format) const = 0;
    virtual void printAsPH (std::ostream& os, const std::string& format) const = 0;

    virtual const double*       getBranchProbs () const = 0;
    virtual const double*       getBranchTransitionMatrix () const = 0;
    virtual int                 getNumOfBranches () const = 0;
    virtual std::vector<double> getBranchMoments (int order) const = 0;
    virtual const int*          getErlangOrders () const = 0;
    virtual const int           getNumOfErlangs () const = 0;
};

#endif // MIXTUREDENSMAP_H
