#ifndef SPECMAP_H
#define SPECMAP_H

#include <vector>
#include <iostream>
#include <cmath>
#include "mixturedensmap.h"

// This class represents an MMAP having the new structure presented in the paper
class SpecMAP : public MixtureDensityMAP {
public:
    int M, N;
    int* branchOrders;
    double* lambda;
    double* alpha;
    double* P;
    double* bigP;

public:

    SpecMAP(std::vector<int> brOrders, int typeNum=1);
    ~SpecMAP();

    void setInitialGuess (InitialGuess initial, Trace* trace, double trMean);

    void updateBigP ();

    void printSummary (std::ostream& os) const;
    void printAsMAP (std::ostream& os, const std::string& format) const;
    void printAsPH (std::ostream& os, const std::string& format) const;

    std::vector<double> getBranchMoments (int order) const;
    const double*       getBranchProbs () const { return alpha; }
    const double*       getBranchTransitionMatrix () const { return bigP; }
    int                 getNumOfBranches () const { return N; }
    void scaleIntensities (double sc);
    const int*          getErlangOrders () const { return branchOrders; }
    const int           getNumOfErlangs () const { return M; }
};

#endif // SPECMAP_H
