#ifndef HEMFIT_H
#define HEMFIT_H

#include "trace.h"
#include "hermap.h"
#include "specmap.h"
#include "febmap.h"
#include <vector>
#include <list>

class SPEMFit {

public:
    enum BranchOrders { All, ProgPres, Incremental };
    enum Structure    { StHErMAP, StSpecMAP, StFEBMAP };

private:
    Trace* trace;
    double trMean;
    std::vector<MixtureDensityMAP*> maps;
    std::vector<std::vector<char> > clustering;
    Structure stru;
    BranchOrders strategy;
    int totalStates;
    InitialGuess initial;

    bool checkForOrder1 (const std::vector<int>& brOrders);
    std::list<std::vector<int> > allOrders (int branches, int sumOrders);
    void genNeighbours (const std::vector<int>& brOrders);

    double refine (MixtureDensityMAP* ini, bool ph, bool da, bool verbose, int maxiter, double precision);

    double refinePH   (FEBMAP* ini, bool verbose, int maxiter, double precision, double beta);
    double refineMMAP (HErMAP* ini, bool verbose, int maxiter, double precision, double beta);
    double refinePH   (HErMAP* ini, bool verbose, int maxiter, double precision, double beta);
    double refineMMAP (SpecMAP* ini, bool verbose, int maxiter, double precision, double beta);
    double refinePH   (SpecMAP* ini, bool verbose, int maxiter, double precision, double beta);

public:
    SPEMFit (Trace* tr);

    void    initFitting (int totalStates, BranchOrders brOrders=ProgPres, Structure stru=StSpecMAP, InitialGuess initial=IGCombined);
    void    initFitting (std::vector<int> brOrders, Structure stru=StSpecMAP, InitialGuess initial=IGCombined);

    MixtureDensityMAP* runFitting (bool ph = false, bool da = true, double eps = 1e-7, int maxIter = 500, bool verbose = false);
};

#endif // HEMFIT_H
