#ifndef STATPROVIDER_H
#define STATPROVIDER_H

# include <vector>

class StatProvider {
public:

    virtual std::vector<double>               getMoments (int upToOrder) const = 0;                   //!< Returns the number of moments requested.
    virtual std::vector<double>               getBackwardMoments (int upToOrder, int k) const = 0;    //!< Returns the moments of the inter-arrival times preceeding a type-k arrival
    virtual std::vector<double>               getForwardMoments  (int upToOrder, int k) const = 0;    //!< Returns the moments of the inter-arrival times following a type-k arrival
    virtual std::vector<std::vector<double> > getJointMoments (int upToOrder) const = 0;              //!< Returns the number of lag-1 joint moments requested.
    virtual std::vector<std::vector<double> > getJointMomentsOfType (int upToOrder, int p) const = 0; //!< Returns the number of lag-1 joint moments requested belonging to type-p arrivals
    virtual std::vector<double>               getAcf (int maxLag) const = 0;                          //!< Returns the lag-k autocorrelation function up to lag maxLag
    virtual std::vector<double>               getCrossAcf (int maxLag, int k, int p) const = 0;       //!< Returns the cross-autocorrelation function between type-k and type-p arrivals up to lag maxLag
};

#endif // STATPROVIDER_H
