#ifndef TRACE_H
#define TRACE_H

#include <vector>
#include <iostream>
#include <map>
#include "statprovider.h"

class Trace : public StatProvider {

    std::vector<double> iatime;
    std::vector<int>    type;
    int numOfTypes;
    bool hasZero;
    std::map<int,std::vector<char> > clusters;

public:
    inline int     size () const { return iatime.size(); }
    inline bool    isSingleType () const { return type.size()==0 && iatime.size()>0; }
           int     getNumOfTypes () const { return numOfTypes; }
    inline double  getIATime (int k) const { return iatime[k]; }
    inline int     getType (int k) const { return type.empty() ? 0 : type[k]; }
    inline bool    hasZeroIATime () const { return hasZero; }
    const std::vector<double>& getIATimes () const { return iatime; }

    std::vector<double>               getMoments (int upToOrder) const;
    std::vector<double>               getBackwardMoments (int upToOrder, int k) const;
    std::vector<double>               getForwardMoments  (int upToOrder, int k) const;
    std::vector<std::vector<double> > getJointMoments (int upToOrder) const;
    std::vector<std::vector<double> > getJointMomentsOfType (int upToOrder, int p) const;
    std::vector<double>               getAcf (int maxLag) const;
    std::vector<double>               getCrossAcf (int maxLag, int k, int p) const;

    std::vector<char> kmeans(int numOfClusters);

    static Trace* loadFromFile (const std::string& fname, int iaFieldNum=0, int typeFieldNum=-1, const std::string& delimiters=" \t;", bool incremental=false, int startLine=0, int endLine=-1);
};
std::ostream& operator<< (std::ostream& out, const Trace& trace);

#endif // TRACE_H
